import csv
import itertools
from typing import TextIO

from .renderers import BaseRenderer, JSONRenderer


class CSVConverter:
    @classmethod
    def convert(
        cls,
        f: TextIO,
        output_format: str,
        skip_rows: int | None = None,
        take_rows: int | None = None,
    ) -> str:
        if skip_rows is not None and take_rows is not None:
            take_rows = skip_rows + take_rows

        rows = [
            row for row in itertools.islice(csv.DictReader(f), skip_rows, take_rows)
        ]

        renderer = cls._get_renderer(output_format)
        rendered_rows = renderer.render(rows)

        return rendered_rows

    @staticmethod
    def _get_renderer(output_format: str) -> BaseRenderer:
        match output_format:
            case "json":
                return JSONRenderer()
            case _:
                raise ValueError(f'The "{output_format}" format is not supported.')


csv_converter = CSVConverter()



import argparse
import sys

from .core import csv_converter


def positive_int_type(value: str) -> int:
    if not value.isdigit():
        raise argparse.ArgumentTypeError("Invalid value.")

    return_value = int(value)

    if return_value < 0:
        raise argparse.ArgumentTypeError("Value may not be less than 0.")

    return return_value


argument_parser = argparse.ArgumentParser(prog="converter")

argument_parser.add_argument(
    "-f",
    "--format",
    dest="output_format",
    choices=["json"],
    default="json",
    help="Determine the format of conversion.",
)

argument_parser.add_argument(
    "-s",
    "--skip",
    dest="skip_rows",
    type=positive_int_type,
    default=None,
    help="Determine how many rows to skip.",
)

argument_parser.add_argument(
    "-t",
    "--take",
    dest="take_rows",
    type=positive_int_type,
    default=None,
    help="Determine how many rows to take.",
)


def main() -> int:
    args = argument_parser.parse_args()
    result = csv_converter.convert(
        sys.stdin,
        output_format=args.output_format,
        skip_rows=args.skip_rows,
        take_rows=args.take_rows,
    )
    sys.stdout.write(result)
    return 0



import abc
import json
from typing import TypeAlias

DictList: TypeAlias = list[dict[str, str]]


class BaseRenderer(abc.ABC):
    @abc.abstractmethod
    def render(self, data: DictList) -> str:
        pass


class JSONRenderer(BaseRenderer):
    def render(self, data: DictList) -> str:
        return json.dumps(data, ensure_ascii=False, indent=4)



import json
from io import StringIO

import pytest

from converter.core import csv_converter


@pytest.fixture
def input_data():
    return StringIO(
        "User,Workstation Type,Serial Number,CPU,RAM,SSD\n"
        "Tom White,Laptop,2889900,AMD Ryzen 7,64GB,10TB\n"
        "John Doe,Laptop,112233,Intel i7,32GB,2TB\n"
        "Barbara Black,PC,556677,Intel i5,16GB,1TB\n"
        "Steve Jones,PC,121212,Intel i3,8GB,512GB"
    )


@pytest.mark.parametrize(
    "skip_rows, take_rows, expected_result",
    [
        (
            None,
            None,
            """
                [
                    {
                        "User": "Tom White",
                        "Workstation Type": "Laptop",
                        "Serial Number": "2889900",
                        "CPU": "AMD Ryzen 7",
                        "RAM": "64GB",
                        "SSD": "10TB"
                    },
                    {
                        "User": "John Doe",
                        "Workstation Type": "Laptop",
                        "Serial Number": "112233",
                        "CPU": "Intel i7",
                        "RAM": "32GB",
                        "SSD": "2TB"
                    },
                    {
                        "User": "Barbara Black",
                        "Workstation Type": "PC",
                        "Serial Number": "556677",
                        "CPU": "Intel i5",
                        "RAM": "16GB",
                        "SSD": "1TB"
                    },
                    {
                        "User": "Steve Jones",
                        "Workstation Type": "PC",
                        "Serial Number": "121212",
                        "CPU": "Intel i3",
                        "RAM": "8GB",
                        "SSD": "512GB"
                    }
                ]
                """,
        ),
        (
            1,
            None,
            """
                [
                    {
                        "User": "John Doe",
                        "Workstation Type": "Laptop",
                        "Serial Number": "112233",
                        "CPU": "Intel i7",
                        "RAM": "32GB",
                        "SSD": "2TB"
                    },
                    {
                        "User": "Barbara Black",
                        "Workstation Type": "PC",
                        "Serial Number": "556677",
                        "CPU": "Intel i5",
                        "RAM": "16GB",
                        "SSD": "1TB"
                    },
                    {
                        "User": "Steve Jones",
                        "Workstation Type": "PC",
                        "Serial Number": "121212",
                        "CPU": "Intel i3",
                        "RAM": "8GB",
                        "SSD": "512GB"
                    }
                ]
                """,
        ),
        (
            2,
            2,
            """
                [
                    {
                        "User": "Barbara Black",
                        "Workstation Type": "PC",
                        "Serial Number": "556677",
                        "CPU": "Intel i5",
                        "RAM": "16GB",
                        "SSD": "1TB"
                    },
                    {
                        "User": "Steve Jones",
                        "Workstation Type": "PC",
                        "Serial Number": "121212",
                        "CPU": "Intel i3",
                        "RAM": "8GB",
                        "SSD": "512GB"
                    }
                ]
                """,
        ),
        (
            2,
            1,
            """
                [
                    {
                        "User": "Barbara Black",
                        "Workstation Type": "PC",
                        "Serial Number": "556677",
                        "CPU": "Intel i5",
                        "RAM": "16GB",
                        "SSD": "1TB"
                    }
                ]
                """,
        ),
    ],
)
def test_converter_with_json_format(input_data, skip_rows, take_rows, expected_result):
    result = csv_converter.convert(
        input_data, output_format="json", skip_rows=skip_rows, take_rows=take_rows
    )
    assert json.loads(result) == json.loads(expected_result)
